@@ -54,11 +54,9 @@ Datum RASTER_summaryStats_finalfn(PG_FUNCTION_ARGS);
 
 /* get histogram */
 Datum RASTER_histogram(PG_FUNCTION_ARGS);
-Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS);
 
 /* get quantiles */
 Datum RASTER_quantile(PG_FUNCTION_ARGS);
-Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS);
 
 /* get counts of values */
 Datum RASTER_valueCount(PG_FUNCTION_ARGS);
@@ -1188,532 +1186,6 @@ Datum RASTER_histogram(PG_FUNCTION_ARGS)
 	}
 }
 
-/**
- * Returns histogram of a coverage for a specified band
- */
-PG_FUNCTION_INFO_V1(RASTER_histogramCoverage);
-Datum RASTER_histogramCoverage(PG_FUNCTION_ARGS)
-{
-	FuncCallContext *funcctx;
-	TupleDesc tupdesc;
-
-	uint32_t i;
-	rt_histogram covhist = NULL;
-	rt_histogram covhist2;
-	int call_cntr;
-	int max_calls;
-
-	POSTGIS_RT_DEBUG(3, "RASTER_histogramCoverage: Starting");
-
-	/* first call of function */
-	if (SRF_IS_FIRSTCALL()) {
-		MemoryContext oldcontext;
-
-		text *tablenametext = NULL;
-		char *tablename = NULL;
-		text *colnametext = NULL;
-		char *colname = NULL;
-		int32_t bandindex = 1;
-		bool exclude_nodata_value = TRUE;
-		double sample = 0;
-		uint32_t bin_count = 0;
-		double *bin_width = NULL;
-		uint32_t bin_width_count = 0;
-		double width = 0;
-		bool right = FALSE;
-		uint32_t count;
-
-		int len = 0;
-		char *sql = NULL;
-		char *tmp = NULL;
-		double min = 0;
-		double max = 0;
-		int spi_result;
-		Portal portal;
-		HeapTuple tuple;
-		Datum datum;
-		bool isNull = FALSE;
-
-		rt_pgraster *pgraster = NULL;
-		rt_raster raster = NULL;
-		rt_band band = NULL;
-		int num_bands = 0;
-		rt_bandstats stats = NULL;
-		rt_histogram hist;
-		uint64_t sum = 0;
-
-		int j;
-		int n;
-
-		ArrayType *array;
-		Oid etype;
-		Datum *e;
-		bool *nulls;
-		int16 typlen;
-		bool typbyval;
-		char typalign;
-
-		POSTGIS_RT_DEBUG(3, "RASTER_histogramCoverage: first call of function");
-
-		/* create a function context for cross-call persistence */
-		funcctx = SRF_FIRSTCALL_INIT();
-
-		/* switch to memory context appropriate for multiple function calls */
-		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
-
-		/* tablename is null, return null */
-		if (PG_ARGISNULL(0)) {
-			elog(NOTICE, "Table name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		tablenametext = PG_GETARG_TEXT_P(0);
-		tablename = text_to_cstring(tablenametext);
-		if (!strlen(tablename)) {
-			elog(NOTICE, "Table name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: tablename = %s", tablename);
-
-		/* column name is null, return null */
-		if (PG_ARGISNULL(1)) {
-			elog(NOTICE, "Column name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		colnametext = PG_GETARG_TEXT_P(1);
-		colname = text_to_cstring(colnametext);
-		if (!strlen(colname)) {
-			elog(NOTICE, "Column name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: colname = %s", colname);
-
-		/* band index is 1-based */
-		if (!PG_ARGISNULL(2))
-			bandindex = PG_GETARG_INT32(2);
-
-		/* exclude_nodata_value flag */
-		if (!PG_ARGISNULL(3))
-			exclude_nodata_value = PG_GETARG_BOOL(3);
-
-		/* sample % */
-		if (!PG_ARGISNULL(4)) {
-			sample = PG_GETARG_FLOAT8(4);
-			if (sample < 0 || sample > 1) {
-				elog(NOTICE, "Invalid sample percentage (must be between 0 and 1). Returning NULL");
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-			else if (FLT_EQ(sample, 0.0))
-				sample = 1;
-		}
-		else
-			sample = 1;
-
-		/* bin_count */
-		if (!PG_ARGISNULL(5)) {
-			bin_count = PG_GETARG_INT32(5);
-			if (bin_count < 1) bin_count = 0;
-		}
-
-		/* bin_width */
-		if (!PG_ARGISNULL(6)) {
-			array = PG_GETARG_ARRAYTYPE_P(6);
-			etype = ARR_ELEMTYPE(array);
-			get_typlenbyvalalign(etype, &typlen, &typbyval, &typalign);
-
-			switch (etype) {
-				case FLOAT4OID:
-				case FLOAT8OID:
-					break;
-				default:
-					MemoryContextSwitchTo(oldcontext);
-					elog(ERROR, "RASTER_histogramCoverage: Invalid data type for width");
-					SRF_RETURN_DONE(funcctx);
-					break;
-			}
-
-			deconstruct_array(array, etype, typlen, typbyval, typalign, &e,
-				&nulls, &n);
-
-			bin_width = palloc(sizeof(double) * n);
-			for (i = 0, j = 0; i < (uint32_t) n; i++) {
-				if (nulls[i]) continue;
-
-				switch (etype) {
-					case FLOAT4OID:
-						width = (double) DatumGetFloat4(e[i]);
-						break;
-					case FLOAT8OID:
-						width = (double) DatumGetFloat8(e[i]);
-						break;
-				}
-
-				if (width < 0 || FLT_EQ(width, 0.0)) {
-					elog(NOTICE, "Invalid value for width (must be greater than 0). Returning NULL");
-					pfree(bin_width);
-					MemoryContextSwitchTo(oldcontext);
-					SRF_RETURN_DONE(funcctx);
-				}
-
-				bin_width[j] = width;
-				POSTGIS_RT_DEBUGF(5, "bin_width[%d] = %f", j, bin_width[j]);
-				j++;
-			}
-			bin_width_count = j;
-
-			if (j < 1) {
-				pfree(bin_width);
-				bin_width = NULL;
-			}
-		}
-
-		/* right */
-		if (!PG_ARGISNULL(7))
-			right = PG_GETARG_BOOL(7);
-
-		/* connect to database */
-		spi_result = SPI_connect();
-		if (spi_result != SPI_OK_CONNECT) {
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot connect to database using SPI");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		/* coverage stats */
-		len = sizeof(char) * (strlen("SELECT min, max FROM _st_summarystats('','',,::boolean,)") + strlen(tablename) + strlen(colname) + (MAX_INT_CHARLEN * 2) + MAX_DBL_CHARLEN + 1);
-		sql = (char *) palloc(len);
-		if (NULL == sql) {
-
-			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-			SPI_finish();
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot allocate memory for sql");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		/* get stats */
-		snprintf(sql, len, "SELECT min, max FROM _st_summarystats('%s','%s',%d,%d::boolean,%f)", tablename, colname, bandindex, (exclude_nodata_value ? 1 : 0), sample);
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: %s", sql);
-		spi_result = SPI_execute(sql, TRUE, 0);
-		pfree(sql);
-		if (spi_result != SPI_OK_SELECT || SPI_tuptable == NULL || SPI_processed != 1) {
-
-			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-			SPI_finish();
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot get summary stats of coverage");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		tupdesc = SPI_tuptable->tupdesc;
-		tuple = SPI_tuptable->vals[0];
-
-		tmp = SPI_getvalue(tuple, tupdesc, 1);
-		if (NULL == tmp || !strlen(tmp)) {
-
-			SPI_freetuptable(SPI_tuptable);
-			SPI_finish();
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot get summary stats of coverage");
-			SRF_RETURN_DONE(funcctx);
-		}
-		min = strtod(tmp, NULL);
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: min = %f", min);
-		pfree(tmp);
-
-		tmp = SPI_getvalue(tuple, tupdesc, 2);
-		if (NULL == tmp || !strlen(tmp)) {
-
-			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-			SPI_finish();
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot get summary stats of coverage");
-			SRF_RETURN_DONE(funcctx);
-		}
-		max = strtod(tmp, NULL);
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: max = %f", max);
-		pfree(tmp);
-
-		/* iterate through rasters of coverage */
-		/* create sql */
-		len = sizeof(char) * (strlen("SELECT \"\" FROM \"\" WHERE \"\" IS NOT NULL") + (strlen(colname) * 2) + strlen(tablename) + 1);
-		sql = (char *) palloc(len);
-		if (NULL == sql) {
-
-			if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-			SPI_finish();
-
-			if (bin_width_count) pfree(bin_width);
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_histogramCoverage: Cannot allocate memory for sql");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		/* get cursor */
-		snprintf(sql, len, "SELECT \"%s\" FROM \"%s\" WHERE \"%s\" IS NOT NULL", colname, tablename, colname);
-		POSTGIS_RT_DEBUGF(3, "RASTER_histogramCoverage: %s", sql);
-		portal = SPI_cursor_open_with_args(
-			"coverage",
-			sql,
-			0, NULL,
-			NULL, NULL,
-			TRUE, 0
-		);
-		pfree(sql);
-
-		/* process resultset */
-		SPI_cursor_fetch(portal, TRUE, 1);
-		while (SPI_processed == 1 && SPI_tuptable != NULL) {
-			tupdesc = SPI_tuptable->tupdesc;
-			tuple = SPI_tuptable->vals[0];
-
-			datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
-			if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				if (NULL != covhist) pfree(covhist);
-				if (bin_width_count) pfree(bin_width);
-
-				MemoryContextSwitchTo(oldcontext);
-				elog(ERROR, "RASTER_histogramCoverage: Cannot get raster of coverage");
-				SRF_RETURN_DONE(funcctx);
-			}
-			else if (isNull) {
-				SPI_cursor_fetch(portal, TRUE, 1);
-				continue;
-			}
-
-			pgraster = (rt_pgraster *) PG_DETOAST_DATUM(datum);
-
-			raster = rt_raster_deserialize(pgraster, FALSE);
-			if (!raster) {
-
-				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				if (NULL != covhist) pfree(covhist);
-				if (bin_width_count) pfree(bin_width);
-
-				MemoryContextSwitchTo(oldcontext);
-				elog(ERROR, "RASTER_histogramCoverage: Cannot deserialize raster");
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* inspect number of bands*/
-			num_bands = rt_raster_get_num_bands(raster);
-			if (bandindex < 1 || bandindex > num_bands) {
-				elog(NOTICE, "Invalid band index (must use 1-based). Returning NULL");
-
-				rt_raster_destroy(raster);
-
-				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				if (NULL != covhist) pfree(covhist);
-				if (bin_width_count) pfree(bin_width);
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* get band */
-			band = rt_raster_get_band(raster, bandindex - 1);
-			if (!band) {
-				elog(NOTICE, "Cannot find band at index %d. Returning NULL", bandindex);
-
-				rt_raster_destroy(raster);
-
-				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				if (NULL != covhist) pfree(covhist);
-				if (bin_width_count) pfree(bin_width);
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* we need the raw values, hence the non-zero parameter */
-			stats = rt_band_get_summary_stats(band, (int) exclude_nodata_value, sample, 1, NULL, NULL, NULL);
-
-			rt_band_destroy(band);
-			rt_raster_destroy(raster);
-
-			if (NULL == stats) {
-				elog(NOTICE, "Cannot compute summary statistics for band at index %d. Returning NULL", bandindex);
-
-				if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				if (NULL != covhist) pfree(covhist);
-				if (bin_width_count) pfree(bin_width);
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* get histogram */
-			if (stats->count > 0) {
-				hist = rt_band_get_histogram(stats, bin_count, bin_width, bin_width_count, right, min, max, &count);
-				pfree(stats);
-				if (NULL == hist || !count) {
-					elog(NOTICE, "Cannot compute histogram for band at index %d", bandindex);
-
-					if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-					SPI_cursor_close(portal);
-					SPI_finish();
-
-					if (NULL != covhist) pfree(covhist);
-					if (bin_width_count) pfree(bin_width);
-
-					MemoryContextSwitchTo(oldcontext);
-					SRF_RETURN_DONE(funcctx);
-				}
-
-				POSTGIS_RT_DEBUGF(3, "%d bins returned", count);
-
-				/* coverage histogram */
-				if (NULL == covhist) {
-					covhist = (rt_histogram) SPI_palloc(sizeof(struct rt_histogram_t) * count);
-					if (NULL == covhist) {
-
-						pfree(hist);
-						if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-						SPI_cursor_close(portal);
-						SPI_finish();
-
-						if (bin_width_count) pfree(bin_width);
-
-						MemoryContextSwitchTo(oldcontext);
-						elog(ERROR, "RASTER_histogramCoverage: Cannot allocate memory for histogram of coverage");
-						SRF_RETURN_DONE(funcctx);
-					}
-
-					for (i = 0; i < count; i++) {
-						sum += hist[i].count;
-						covhist[i].count = hist[i].count;
-						covhist[i].percent = 0;
-						covhist[i].min = hist[i].min;
-						covhist[i].max = hist[i].max;
-					}
-				}
-				else {
-					for (i = 0; i < count; i++) {
-						sum += hist[i].count;
-						covhist[i].count += hist[i].count;
-					}
-				}
-
-				pfree(hist);
-
-				/* assuming bin_count wasn't set, force consistency */
-				if (bin_count <= 0) bin_count = count;
-			}
-
-			/* next record */
-			SPI_cursor_fetch(portal, TRUE, 1);
-		}
-
-		if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-		SPI_cursor_close(portal);
-		SPI_finish();
-
-		if (bin_width_count) pfree(bin_width);
-
-		/* finish percent of histogram */
-		if (sum > 0) {
-			for (i = 0; i < count; i++)
-				covhist[i].percent = covhist[i].count / (double) sum;
-		}
-
-		/* Store needed information */
-		funcctx->user_fctx = covhist;
-
-		/* total number of tuples to be returned */
-		funcctx->max_calls = count;
-
-		/* Build a tuple descriptor for our result type */
-		if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE) {
-			ereport(ERROR, (
-				errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				errmsg(
-					"function returning record called in context "
-					"that cannot accept type record"
-				)
-			));
-		}
-
-		BlessTupleDesc(tupdesc);
-		funcctx->tuple_desc = tupdesc;
-
-		MemoryContextSwitchTo(oldcontext);
-	}
-
-	/* stuff done on every call of the function */
-	funcctx = SRF_PERCALL_SETUP();
-
-	call_cntr = funcctx->call_cntr;
-	max_calls = funcctx->max_calls;
-	tupdesc = funcctx->tuple_desc;
-	covhist2 = funcctx->user_fctx;
-
-	/* do when there is more left to send */
-	if (call_cntr < max_calls) {
-		Datum values[VALUES_LENGTH];
-		bool nulls[VALUES_LENGTH];
-		HeapTuple tuple;
-		Datum result;
-
-		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
-
-		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
-
-		values[0] = Float8GetDatum(covhist2[call_cntr].min);
-		values[1] = Float8GetDatum(covhist2[call_cntr].max);
-		values[2] = Int64GetDatum(covhist2[call_cntr].count);
-		values[3] = Float8GetDatum(covhist2[call_cntr].percent);
-
-		/* build a tuple */
-		tuple = heap_form_tuple(tupdesc, values, nulls);
-
-		/* make the tuple into a datum */
-		result = HeapTupleGetDatum(tuple);
-
-		SRF_RETURN_NEXT(funcctx, result);
-	}
-	/* do when there is no more left */
-	else {
-		pfree(covhist2);
-		SRF_RETURN_DONE(funcctx);
-	}
-}
-
 #undef VALUES_LENGTH
 #define VALUES_LENGTH 2
 
@@ -1966,436 +1438,6 @@ Datum RASTER_quantile(PG_FUNCTION_ARGS)
 	}
 }
 
-/**
- * Returns selected quantiles of a coverage for a specified band
- */
-PG_FUNCTION_INFO_V1(RASTER_quantileCoverage);
-Datum RASTER_quantileCoverage(PG_FUNCTION_ARGS)
-{
-	FuncCallContext *funcctx;
-	TupleDesc tupdesc;
-
-	uint32_t i;
-	rt_quantile covquant = NULL;
-	rt_quantile covquant2;
-	int call_cntr;
-	int max_calls;
-
-	POSTGIS_RT_DEBUG(3, "RASTER_quantileCoverage: Starting");
-
-	/* first call of function */
-	if (SRF_IS_FIRSTCALL()) {
-		MemoryContext oldcontext;
-
-		text *tablenametext = NULL;
-		char *tablename = NULL;
-		text *colnametext = NULL;
-		char *colname = NULL;
-		int32_t bandindex = 1;
-		bool exclude_nodata_value = TRUE;
-		double sample = 0;
-		double *quantiles = NULL;
-		uint32_t quantiles_count = 0;
-		double quantile = 0;
-		uint32_t count;
-
-		int len = 0;
-		char *sql = NULL;
-		char *tmp = NULL;
-		uint64_t cov_count = 0;
-		int spi_result;
-		Portal portal;
-		SPITupleTable *tuptable = NULL;
-		HeapTuple tuple;
-		Datum datum;
-		bool isNull = FALSE;
-
-		rt_pgraster *pgraster = NULL;
-		rt_raster raster = NULL;
-		rt_band band = NULL;
-		int num_bands = 0;
-		struct quantile_llist *qlls = NULL;
-		uint32_t qlls_count;
-
-		int j;
-		int n;
-
-		ArrayType *array;
-		Oid etype;
-		Datum *e;
-		bool *nulls;
-		int16 typlen;
-		bool typbyval;
-		char typalign;
-
-		POSTGIS_RT_DEBUG(3, "RASTER_quantileCoverage: first call of function");
-
-		/* create a function context for cross-call persistence */
-		funcctx = SRF_FIRSTCALL_INIT();
-
-		/* switch to memory context appropriate for multiple function calls */
-		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
-
-		/* tablename is null, return null */
-		if (PG_ARGISNULL(0)) {
-			elog(NOTICE, "Table name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		tablenametext = PG_GETARG_TEXT_P(0);
-		tablename = text_to_cstring(tablenametext);
-		if (!strlen(tablename)) {
-			elog(NOTICE, "Table name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		POSTGIS_RT_DEBUGF(3, "RASTER_quantileCoverage: tablename = %s", tablename);
-
-		/* column name is null, return null */
-		if (PG_ARGISNULL(1)) {
-			elog(NOTICE, "Column name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		colnametext = PG_GETARG_TEXT_P(1);
-		colname = text_to_cstring(colnametext);
-		if (!strlen(colname)) {
-			elog(NOTICE, "Column name must be provided");
-			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
-		}
-		POSTGIS_RT_DEBUGF(3, "RASTER_quantileCoverage: colname = %s", colname);
-
-		/* band index is 1-based */
-		if (!PG_ARGISNULL(2))
-			bandindex = PG_GETARG_INT32(2);
-
-		/* exclude_nodata_value flag */
-		if (!PG_ARGISNULL(3))
-			exclude_nodata_value = PG_GETARG_BOOL(3);
-
-		/* sample % */
-		if (!PG_ARGISNULL(4)) {
-			sample = PG_GETARG_FLOAT8(4);
-			if (sample < 0 || sample > 1) {
-				elog(NOTICE, "Invalid sample percentage (must be between 0 and 1). Returning NULL");
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-			else if (FLT_EQ(sample, 0.0))
-				sample = 1;
-		}
-		else
-			sample = 1;
-
-		/* quantiles */
-		if (!PG_ARGISNULL(5)) {
-			array = PG_GETARG_ARRAYTYPE_P(5);
-			etype = ARR_ELEMTYPE(array);
-			get_typlenbyvalalign(etype, &typlen, &typbyval, &typalign);
-
-			switch (etype) {
-				case FLOAT4OID:
-				case FLOAT8OID:
-					break;
-				default:
-					MemoryContextSwitchTo(oldcontext);
-					elog(ERROR, "RASTER_quantileCoverage: Invalid data type for quantiles");
-					SRF_RETURN_DONE(funcctx);
-					break;
-			}
-
-			deconstruct_array(array, etype, typlen, typbyval, typalign, &e,
-				&nulls, &n);
-
-			quantiles = palloc(sizeof(double) * n);
-			for (i = 0, j = 0; i < (uint32_t) n; i++) {
-				if (nulls[i]) continue;
-
-				switch (etype) {
-					case FLOAT4OID:
-						quantile = (double) DatumGetFloat4(e[i]);
-						break;
-					case FLOAT8OID:
-						quantile = (double) DatumGetFloat8(e[i]);
-						break;
-				}
-
-				if (quantile < 0 || quantile > 1) {
-					elog(NOTICE, "Invalid value for quantile (must be between 0 and 1). Returning NULL");
-					pfree(quantiles);
-					MemoryContextSwitchTo(oldcontext);
-					SRF_RETURN_DONE(funcctx);
-				}
-
-				quantiles[j] = quantile;
-				POSTGIS_RT_DEBUGF(5, "quantiles[%d] = %f", j, quantiles[j]);
-				j++;
-			}
-			quantiles_count = j;
-
-			if (j < 1) {
-				pfree(quantiles);
-				quantiles = NULL;
-			}
-		}
-
-		/* coverage stats */
-		/* connect to database */
-		spi_result = SPI_connect();
-		if (spi_result != SPI_OK_CONNECT) {
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_quantileCoverage: Cannot connect to database using SPI");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		len = sizeof(char) * (strlen("SELECT count FROM _st_summarystats('','',,::boolean,)") + strlen(tablename) + strlen(colname) + (MAX_INT_CHARLEN * 2) + MAX_DBL_CHARLEN + 1);
-		sql = (char *) palloc(len);
-		if (NULL == sql) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
-			SPI_finish();
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_quantileCoverage: Cannot allocate memory for sql");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		/* get stats */
-		snprintf(sql, len, "SELECT count FROM _st_summarystats('%s','%s',%d,%d::boolean,%f)", tablename, colname, bandindex, (exclude_nodata_value ? 1 : 0), sample);
-		POSTGIS_RT_DEBUGF(3, "stats sql:  %s", sql);
-		spi_result = SPI_execute(sql, TRUE, 0);
-		pfree(sql);
-		if (spi_result != SPI_OK_SELECT || SPI_tuptable == NULL || SPI_processed != 1) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
-			SPI_finish();
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_quantileCoverage: Cannot get summary stats of coverage");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		tupdesc = SPI_tuptable->tupdesc;
-		tuptable = SPI_tuptable;
-		tuple = tuptable->vals[0];
-
-		tmp = SPI_getvalue(tuple, tupdesc, 1);
-		if (NULL == tmp || !strlen(tmp)) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
-			SPI_finish();
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_quantileCoverage: Cannot get summary stats of coverage");
-			SRF_RETURN_DONE(funcctx);
-		}
-		cov_count = strtol(tmp, NULL, 10);
-		POSTGIS_RT_DEBUGF(3, "covcount = %d", (int) cov_count);
-		pfree(tmp);
-
-		/* iterate through rasters of coverage */
-		/* create sql */
-		len = sizeof(char) * (strlen("SELECT \"\" FROM \"\" WHERE \"\" IS NOT NULL") + (strlen(colname) * 2) + strlen(tablename) + 1);
-		sql = (char *) palloc(len);
-		if (NULL == sql) {
-
-			if (SPI_tuptable) SPI_freetuptable(tuptable);
-			SPI_finish();
-
-			MemoryContextSwitchTo(oldcontext);
-			elog(ERROR, "RASTER_quantileCoverage: Cannot allocate memory for sql");
-			SRF_RETURN_DONE(funcctx);
-		}
-
-		/* get cursor */
-		snprintf(sql, len, "SELECT \"%s\" FROM \"%s\" WHERE \"%s\" IS NOT NULL", colname, tablename, colname);
-		POSTGIS_RT_DEBUGF(3, "coverage sql: %s", sql);
-		portal = SPI_cursor_open_with_args(
-			"coverage",
-			sql,
-			0, NULL,
-			NULL, NULL,
-			TRUE, 0
-		);
-		pfree(sql);
-
-		/* process resultset */
-		SPI_cursor_fetch(portal, TRUE, 1);
-		while (SPI_processed == 1 && SPI_tuptable != NULL) {
-			if (NULL != covquant) pfree(covquant);
-
-			tupdesc = SPI_tuptable->tupdesc;
-			tuple = SPI_tuptable->vals[0];
-
-			datum = SPI_getbinval(tuple, tupdesc, 1, &isNull);
-			if (SPI_result == SPI_ERROR_NOATTRIBUTE) {
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				MemoryContextSwitchTo(oldcontext);
-				elog(ERROR, "RASTER_quantileCoverage: Cannot get raster of coverage");
-				SRF_RETURN_DONE(funcctx);
-			}
-			else if (isNull) {
-				SPI_cursor_fetch(portal, TRUE, 1);
-				continue;
-			}
-
-			pgraster = (rt_pgraster *) PG_DETOAST_DATUM(datum);
-
-			raster = rt_raster_deserialize(pgraster, FALSE);
-			if (!raster) {
-
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				MemoryContextSwitchTo(oldcontext);
-				elog(ERROR, "RASTER_quantileCoverage: Cannot deserialize raster");
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* inspect number of bands*/
-			num_bands = rt_raster_get_num_bands(raster);
-			if (bandindex < 1 || bandindex > num_bands) {
-				elog(NOTICE, "Invalid band index (must use 1-based). Returning NULL");
-
-				rt_raster_destroy(raster);
-
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* get band */
-			band = rt_raster_get_band(raster, bandindex - 1);
-			if (!band) {
-				elog(NOTICE, "Cannot find raster band of index %d. Returning NULL", bandindex);
-
-				rt_raster_destroy(raster);
-
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			covquant = rt_band_get_quantiles_stream(
-				band,
-				exclude_nodata_value, sample, cov_count,
-				&qlls, &qlls_count,
-				quantiles, quantiles_count,
-				&count
-			);
-
-			rt_band_destroy(band);
-			rt_raster_destroy(raster);
-
-			if (!covquant || !count) {
-				elog(NOTICE, "Cannot compute quantiles for band at index %d", bandindex);
-
-				SPI_freetuptable(SPI_tuptable);
-				SPI_cursor_close(portal);
-				SPI_finish();
-
-				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
-			}
-
-			/* next record */
-			SPI_cursor_fetch(portal, TRUE, 1);
-		}
-
-		covquant2 = SPI_palloc(sizeof(struct rt_quantile_t) * count);
-		for (i = 0; i < count; i++) {
-			covquant2[i].quantile = covquant[i].quantile;
-			covquant2[i].has_value = covquant[i].has_value;
-			if (covquant2[i].has_value)
-				covquant2[i].value = covquant[i].value;
-		}
-
-		pfree(covquant);
-		quantile_llist_destroy(&qlls, qlls_count);
-
-		if (SPI_tuptable) SPI_freetuptable(SPI_tuptable);
-		SPI_cursor_close(portal);
-		SPI_finish();
-
-		if (quantiles_count) pfree(quantiles);
-
-		POSTGIS_RT_DEBUGF(3, "%d quantiles returned", count);
-
-		/* Store needed information */
-		funcctx->user_fctx = covquant2;
-
-		/* total number of tuples to be returned */
-		funcctx->max_calls = count;
-
-		/* Build a tuple descriptor for our result type */
-		if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE) {
-			ereport(ERROR, (
-				errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				errmsg(
-					"function returning record called in context "
-					"that cannot accept type record"
-				)
-			));
-		}
-
-		BlessTupleDesc(tupdesc);
-		funcctx->tuple_desc = tupdesc;
-
-		MemoryContextSwitchTo(oldcontext);
-	}
-
-	/* stuff done on every call of the function */
-	funcctx = SRF_PERCALL_SETUP();
-
-	call_cntr = funcctx->call_cntr;
-	max_calls = funcctx->max_calls;
-	tupdesc = funcctx->tuple_desc;
-	covquant2 = funcctx->user_fctx;
-
-	/* do when there is more left to send */
-	if (call_cntr < max_calls) {
-		Datum values[VALUES_LENGTH];
-		bool nulls[VALUES_LENGTH];
-		HeapTuple tuple;
-		Datum result;
-
-		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
-
-		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
-
-		values[0] = Float8GetDatum(covquant2[call_cntr].quantile);
-		if (covquant2[call_cntr].has_value)
-			values[1] = Float8GetDatum(covquant2[call_cntr].value);
-		else
-			nulls[1] = TRUE;
-
-		/* build a tuple */
-		tuple = heap_form_tuple(tupdesc, values, nulls);
-
-		/* make the tuple into a datum */
-		result = HeapTupleGetDatum(tuple);
-
-		SRF_RETURN_NEXT(funcctx, result);
-	}
-	/* do when there is no more left */
-	else {
-		POSTGIS_RT_DEBUG(3, "done");
-		pfree(covquant2);
-		SRF_RETURN_DONE(funcctx);
-	}
-}
-
 #undef VALUES_LENGTH
 #define VALUES_LENGTH 3
 
@@ -3031,7 +2073,7 @@ Datum RASTER_valueCountCoverage(PG_FUNCTION_ARGS) {
 	}
 	/* do when there is no more left */
 	else {
-		pfree(covvcnts2);
+		if (covvcnts2) pfree(covvcnts2);
 		SRF_RETURN_DONE(funcctx);
 	}
 }
